Package: ShortForm
Type: Package
Title: Automatic Short Form Creation
Version: 0.5.8
Date: 2026-05-06
Authors@R: c(person("Anthony", "Raborn", email = "anthony.w.raborn@gmail.com", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-8083-4739")), 
    person("Walter", "Leite", email = "Walter.Leite@coe.ufl.edu", role = "aut"))
Description: Performs automatic creation of short forms of scales with an 
    ant colony optimization algorithm and a Tabu search. As implemented in the 
    package, the ant colony algorithm randomly selects items to build a model of 
    a specified length, then updates the probability of item selection according 
    to the fit of the best model within each set of searches. The algorithm 
    continues until the same items are selected by multiple ants a given number 
    of times in a row. On the other hand, the Tabu search changes one parameter at
    a time to be either free, constrained, or fixed while keeping track of the
    changes made and putting changes that result in worse fit in a "tabu" list
    so that the algorithm does not revisit them for some number of searches. 
    See Leite, Huang, & Marcoulides (2008) <doi:10.1080/00273170802285743> for
    an applied example of the ant colony algorithm, and Marcoulides & Falk (2018)
    <doi:10.1080/10705511.2017.1409074> for an applied example of the Tabu search.
License: LGPL (>= 2.0, < 3) | Mozilla Public License
LazyData: TRUE
Suggests: knitr, MplusAutomation (>= 0.7), rmarkdown, testthat
Imports: lavaan (>= 0.5-22), stringr, methods, doSNOW, parallel,
        foreach, rlang
Depends: R (>= 4.1.0)
URL: https://github.com/AnthonyRaborn/ShortForm
BugReports: https://github.com/AnthonyRaborn/ShortForm/issues
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-06 05:13:18 UTC; araborn
Author: Anthony Raborn [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8083-4739>),
  Walter Leite [aut]
Maintainer: Anthony Raborn <anthony.w.raborn@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-06 07:20:45 UTC
Built: R 4.5.2; ; 2026-05-06 11:32:15 UTC; unix
